package BIDM::Tagset;

=head1 NAME

MTk::BIDMTagset

=head1 DESCRIPTION

A BIDMTagset holds the set of tags parsed from an Asset's html representation.

=cut

use strict;
use vars qw(@ISA);

use BIDM::MemberMixin;

@ISA = qw(BIDM::MemberMixin);

sub new
{
    my($class) = @_;
    my($self);

    $self = {
	tags => [],
    };

    bless($self, $class);

    return $self;
}

=head2 $tagset->add_tag($tag)

Add a tag to the tag set.

=cut

sub add_tag
{
    my($self, $tag) = @_;

    push(@{$self->{tags}}, $tag);
}

=head2 $tagset->parse()

Parse the tagset into a RIG object hierarchy.
    
Returns the BIDM::RIGObject subclass representing this object.
    
We assume currently that there is one RIG object per tagset (and one 
tagset per html page). 

Approach: iterate through the tag list, passing to each the current
object that we've parsed out. When we start, the current object is
undefined. The parsing of the first tag will cause the current object
to be defined. 

=cut

sub parse
{
    my($self) = @_;
    my($tag, $obj);

    for $tag (@{$self->{tags}})
    {
	$obj = $tag->parse($obj);
    }
    
    return $obj;
}

=head2 $tagset->store_in_db($db, $id)

Store the tagset in the Tag table in the given database. $id is the
asset id.
    
=cut

sub store_in_db
{
    my($self, $db, $id) = @_;
    my($tag, $tagnum);

    $tagnum = 0;
    
    for $tag (@{$self->{tags}})
    {
	$tag->store_in_db($db, $id, $tagnum);
	$tagnum++;
    }
}

=head2 $tagset->load_from_db($db, $id)

Load the tags for asset $id from the database. Create the BIDM::Tag
objects for each and store in this tagset. 

=cut

sub load_from_db
{
    my($self, $db, $id) = @_;
    my($tag, $tagnum, %tags, $type, $attrs);
    my($q, $k, $v, $l);

    $q = $db->query("select tag_number, tag_type, attr_key, attr_value from Tag where asset_id = '$id'");
    if (!$q)
    {
	#print "Query failed: ", $db->error(), "\n";
	return undef;
    }

    for $l (@{$q->row_lists()})
    {
	($tagnum, $type, $k, $v) = @$l;
#	print "Got <$tagnum> <$type> <$k> <$v>\n";
	$tags{$tagnum}->[1]->{$k} = $v;
	$tags{$tagnum}->[0] = $type;
    }
    for $tagnum (keys(%tags))
    {
	$type = $tags{$tagnum}->[0];
	$attrs = $tags{$tagnum}->[1];
# 	print "Tag $tagnum type=$type attrs:\n";
# 	while (($k, $v) = each(%$attrs))
# 	{
# 	    print "  $k = $v\n";
# 	}
	$tag = BIDM::Tag::create_from_tag($type, $attrs);
	$self->add_tag($tag);
    }
}
    

sub print
{
    my($self, $fh) = @_;
    my($tag);
    
    $fh = \*STDOUT unless defined($fh);

    print $fh "Tagset $self: \n";
    for $tag (@{$self->{tags}})
    {
	$tag->print($fh);
    }
    
}

=head2 $tagset->response($val)

Set or get the response member. This is used to retain the HTTP headers
from the retrieval of this tagset from the web.

=cut

sub response { shift->_elem('response', @_); }

1;
